searchData={"items":[{"type":"module","title":"Floki","doc":"Floki is a simple HTML parser that enables search for nodes using CSS selectors.","ref":"Floki.html"},{"type":"module","title":"Example - Floki","doc":"Assuming that you have the following HTML:\n\n```html\n<!doctype html>\n \n \n   \n     Floki \n     Github page \n     philss \n   \n \n \n```\n\nTo parse this, you can use the function `Floki.parse_document/1`:\n\n```elixir\n{:ok, html} = Floki.parse_document(doc)\n# =>\n# [{\"html\", [],\n#   [\n#     {\"body\", [],\n#      [\n#        {\"section\", [{\"id\", \"content\"}],\n#         [\n#           {\"p\", [{\"class\", \"headline\"}], [\"Floki\"]},\n#           {\"a\", [{\"href\", \"http://github.com/philss/floki\"}], [\"Github page\"]},\n#           {\"span\", [{\"data-model\", \"user\"}], [\"philss\"]}\n#         ]}\n#      ]}\n#   ]}]\n```\n\nWith this document you can perform queries such as:\n\n  * `Floki.find(html, \"#content\")`\n  * `Floki.find(html, \".headline\")`\n  * `Floki.find(html, \"a\")`\n  * `Floki.find(html, \"[data-model=user]\")`\n  * `Floki.find(html, \"#content a\")`\n  * `Floki.find(html, \".headline, a\")`\n\nEach HTML node is represented by a tuple like:\n\n    {tag_name, attributes, children_nodes}\n\nExample of node:\n\n    {\"p\", [{\"class\", \"headline\"}], [\"Floki\"]}\n\nSo even if the only child node is the element text, it is represented\ninside a list.","ref":"Floki.html#module-example"},{"type":"function","title":"Floki.attr/4","doc":"Changes the attribute values of the elements matched by `selector`\nwith the function `mutation` and returns the whole element tree.","ref":"Floki.html#attr/4"},{"type":"function","title":"Examples - Floki.attr/4","doc":"iex> Floki.attr([{\"div\", [{\"id\", \"a\"}], []}], \"#a\", \"id\", fn(id) -> String.replace(id, \"a\", \"b\") end)\n    [{\"div\", [{\"id\", \"b\"}], []}]\n\n    iex> Floki.attr([{\"div\", [{\"class\", \"name\"}], []}], \"div\", \"id\", fn _ -> \"b\" end)\n    [{\"div\", [{\"id\", \"b\"}, {\"class\", \"name\"}], []}]","ref":"Floki.html#attr/4-examples"},{"type":"function","title":"Floki.attribute/2","doc":"Returns a list with attribute values from elements.","ref":"Floki.html#attribute/2"},{"type":"function","title":"Examples - Floki.attribute/2","doc":"iex> Floki.attribute([{\"a\", [{\"href\", \"https://google.com\"}], [\"Google\"]}], \"href\")\n    [\"https://google.com\"]\n\n    iex> Floki.attribute([{\"a\", [{\"href\", \"https://google.com\"}, {\"data-name\", \"google\"}], [\"Google\"]}], \"data-name\")\n    [\"google\"]","ref":"Floki.html#attribute/2-examples"},{"type":"function","title":"Floki.attribute/3","doc":"Returns a list with attribute values for a given selector.","ref":"Floki.html#attribute/3"},{"type":"function","title":"Examples - Floki.attribute/3","doc":"iex> Floki.attribute([{\"a\", [{\"href\", \"https://google.com\"}], [\"Google\"]}], \"a\", \"href\")\n    [\"https://google.com\"]\n\n    iex> Floki.attribute(\n    iex>   [{\"a\", [{\"class\", \"foo\"}, {\"href\", \"https://google.com\"}], [\"Google\"]}],\n    iex>   \"a\",\n    iex>   \"class\"\n    iex> )\n    [\"foo\"]\n\n    iex> Floki.attribute(\n    iex>   [{\"a\", [{\"href\", \"https://e.corp.com\"}, {\"data-name\", \"e.corp\"}], [\"E.Corp\"]}],\n    iex>   \"a[data-name]\",\n    iex>   \"data-name\"\n    iex> )\n    [\"e.corp\"]","ref":"Floki.html#attribute/3-examples"},{"type":"function","title":"Floki.children/2","doc":"Returns the direct child nodes of a HTML node.\n\nBy default, it will also include all texts. You can disable\nthis behaviour by using the option `include_text` to `false`.\n\nIf the given node is not an HTML tag, then it returns nil.","ref":"Floki.html#children/2"},{"type":"function","title":"Examples - Floki.children/2","doc":"iex> Floki.children({\"div\", [], [\"text\", {\"span\", [], []}]})\n    [\"text\", {\"span\", [], []}]\n\n    iex> Floki.children({\"div\", [], [\"text\", {\"span\", [], []}]}, include_text: false)\n    [{\"span\", [], []}]\n\n    iex> Floki.children({:comment, \"comment\"})\n    nil","ref":"Floki.html#children/2-examples"},{"type":"function","title":"Floki.css_escape/1","doc":"Escapes a string for use as a CSS identifier.","ref":"Floki.html#css_escape/1"},{"type":"function","title":"Examples - Floki.css_escape/1","doc":"iex> Floki.css_escape(\"hello world\")\n    \"hello\\\\ world\"\n\n    iex> Floki.css_escape(\"-123\")\n    \"-\\\\31 23\"","ref":"Floki.html#css_escape/1-examples"},{"type":"type","title":"Floki.css_selector/0","doc":"","ref":"Floki.html#t:css_selector/0"},{"type":"function","title":"Floki.filter_out/2","doc":"Returns the nodes from a HTML tree that don't match the filter selector.","ref":"Floki.html#filter_out/2"},{"type":"function","title":"Examples - Floki.filter_out/2","doc":"iex> Floki.filter_out({\"div\", [], [{\"script\", [], [\"hello\"]}, \" world\"]}, \"script\")\n    {\"div\", [], [\" world\"]}\n\n    iex> Floki.filter_out([{\"body\", [], [{\"script\", [], []}, {\"div\", [], []}]}], \"script\")\n    [{\"body\", [], [{\"div\", [], []}]}]\n\n    iex> Floki.filter_out({\"div\", [], [{:comment, \"comment\"}, \" text\"]}, :comment)\n    {\"div\", [], [\" text\"]}\n\n    iex> Floki.filter_out({\"div\", [], [\"text\"]}, :text)\n    {\"div\", [], []}","ref":"Floki.html#filter_out/2-examples"},{"type":"function","title":"Floki.find/2","doc":"Find elements inside an HTML tree or string.","ref":"Floki.html#find/2"},{"type":"function","title":"Examples - Floki.find/2","doc":"iex> {:ok, html} = Floki.parse_fragment(\"  hello  \")\n    iex> Floki.find(html, \".hint\")\n    [{\"span\", [{\"class\", \"hint\"}], [\"hello\"]}]\n\n    iex> {:ok, html} = Floki.parse_fragment(\"  Content  \")\n    iex> Floki.find(html, \"#important\")\n    [{\"div\", [{\"id\", \"important\"}], [{\"div\", [], [\"Content\"]}]}]\n\n    iex> {:ok, html} = Floki.parse_fragment(\"  Google  \")\n    iex> Floki.find(html, \"a\")\n    [{\"a\", [{\"href\", \"https://google.com\"}], [\"Google\"]}]\n\n    iex> Floki.find([{ \"div\", [], [{\"a\", [{\"href\", \"https://google.com\"}], [\"Google\"]}]}], \"div a\")\n    [{\"a\", [{\"href\", \"https://google.com\"}], [\"Google\"]}]","ref":"Floki.html#find/2-examples"},{"type":"function","title":"Floki.find_and_update/3","doc":"Searches for elements inside the HTML tree and update those that matches the selector.\n\nIt will return the updated HTML tree.\n\nThis function works in a way similar to `traverse_and_update`, but instead of updating\nthe children nodes, it will only updates the `tag` and `attributes` of the matching nodes.\n\nIf `fun` returns `:delete`, the HTML node will be removed from the tree.","ref":"Floki.html#find_and_update/3"},{"type":"function","title":"Examples - Floki.find_and_update/3","doc":"iex> Floki.find_and_update([{\"a\", [{\"href\", \"http://elixir-lang.com\"}], [\"Elixir\"]}], \"a\", fn\n    iex>   {\"a\", [{\"href\", href}]} ->\n    iex>     {\"a\", [{\"href\", String.replace(href, \"http://\", \"https://\")}]}\n    iex>   other ->\n    iex>     other\n    iex> end)\n    [{\"a\", [{\"href\", \"https://elixir-lang.com\"}], [\"Elixir\"]}]","ref":"Floki.html#find_and_update/3-examples"},{"type":"function","title":"Floki.get_by_id/2","doc":"Finds the first element in an HTML tree by id.\n\nReturns `nil` if no element is found.\n\nThis is useful when there are IDs that contain special characters that\nare invalid when passed as is as a CSS selector.\nIt is similar to the `getElementById` method in the browser.","ref":"Floki.html#get_by_id/2"},{"type":"function","title":"Examples - Floki.get_by_id/2","doc":"iex> {:ok, html} = Floki.parse_fragment(~s[  hello  ])\n    iex> Floki.get_by_id(html, \"id?foo_special:chars\")\n    {\"span\", [{\"class\", \"hint\"}, {\"id\", \"id?foo_special:chars\"}], [\"hello\"]}\n    iex> Floki.get_by_id(html, \"does-not-exist\")\n    nil","ref":"Floki.html#get_by_id/2-examples"},{"type":"type","title":"Floki.html_attribute/0","doc":"","ref":"Floki.html#t:html_attribute/0"},{"type":"type","title":"Floki.html_attributes/0","doc":"","ref":"Floki.html#t:html_attributes/0"},{"type":"type","title":"Floki.html_attributes_map/0","doc":"","ref":"Floki.html#t:html_attributes_map/0"},{"type":"type","title":"Floki.html_comment/0","doc":"","ref":"Floki.html#t:html_comment/0"},{"type":"type","title":"Floki.html_declaration/0","doc":"","ref":"Floki.html#t:html_declaration/0"},{"type":"type","title":"Floki.html_doctype/0","doc":"","ref":"Floki.html#t:html_doctype/0"},{"type":"type","title":"Floki.html_node/0","doc":"","ref":"Floki.html#t:html_node/0"},{"type":"type","title":"Floki.html_tag/0","doc":"","ref":"Floki.html#t:html_tag/0"},{"type":"type","title":"Floki.html_text/0","doc":"","ref":"Floki.html#t:html_text/0"},{"type":"type","title":"Floki.html_tree/0","doc":"","ref":"Floki.html#t:html_tree/0"},{"type":"macro","title":"Floki.is_html_node/1","doc":"","ref":"Floki.html#is_html_node/1"},{"type":"function","title":"Floki.map/2","doc":"","ref":"Floki.html#map/2"},{"type":"function","title":"Floki.parse/1","doc":"Parses a HTML Document from a String.\n\nThe expect string is a valid HTML, but the parser will try\nto parse even with errors.","ref":"Floki.html#parse/1"},{"type":"function","title":"Floki.parse_document/2","doc":"Parses an HTML document from a string.\n\nThis is the main function to get a tree from an HTML string.","ref":"Floki.html#parse_document/2"},{"type":"function","title":"Options - Floki.parse_document/2","doc":"* `:attributes_as_maps` - Change the behaviour of the parser to return the attributes\n    as maps, instead of a list of `{\"key\", \"value\"}`. Default to `false`.\n\n  * `:html_parser` - The module of the backend that is responsible for parsing\n    the HTML string. By default it is set to the built-in parser, and the module\n    name is equal to `Floki.HTMLParser.Mochiweb`, or from the value of the\n    application env of the same name.\n\n    See https://github.com/philss/floki#alternative-html-parsers for more details.\n\n  * `:parser_args` - A list of options to the parser. This can be used to pass options\n    that are specific for a given parser. Defaults to an empty list.","ref":"Floki.html#parse_document/2-options"},{"type":"function","title":"Examples - Floki.parse_document/2","doc":"iex> Floki.parse_document(\"    hello  \")\n    {:ok, [{\"html\", [], [{\"head\", [], []}, {\"body\", [], [\"hello\"]}]}]}\n\n    iex> Floki.parse_document(\"    hello  \", html_parser: Floki.HTMLParser.Mochiweb)\n    {:ok, [{\"html\", [], [{\"head\", [], []}, {\"body\", [], [\"hello\"]}]}]}\n\n    iex> Floki.parse_document(\n    ...>   \"    hello  \",\n    ...>   attributes_as_maps: true,\n    ...>   html_parser: Floki.HTMLParser.Mochiweb\n    ...>)\n    {:ok, [{\"html\", %{}, [{\"head\", %{}, []}, {\"body\", %{\"class\" => \"main\"}, [\"hello\"]}]}]}","ref":"Floki.html#parse_document/2-examples"},{"type":"function","title":"Floki.parse_document!/2","doc":"Parses a HTML Document from a string.\n\nSimilar to `Floki.parse_document/1`, but raises `Floki.ParseError` if there was an\nerror parsing the document.","ref":"Floki.html#parse_document!/2"},{"type":"function","title":"Example - Floki.parse_document!/2","doc":"iex> Floki.parse_document!(\"    hello  \")\n    [{\"html\", [], [{\"head\", [], []}, {\"body\", [], [\"hello\"]}]}]","ref":"Floki.html#parse_document!/2-example"},{"type":"function","title":"Floki.parse_fragment/2","doc":"Parses an HTML fragment from a string.\n\nThis is mostly for parsing sections of an HTML document.","ref":"Floki.html#parse_fragment/2"},{"type":"function","title":"Options - Floki.parse_fragment/2","doc":"* `:attributes_as_maps` - Change the behaviour of the parser to return the attributes\n    as maps, instead of a list of `{\"key\", \"value\"}`. Remember that maps are no longer\n    ordered since OTP 26. Default to `false`.\n\n  * `:html_parser` - The module of the backend that is responsible for parsing\n    the HTML string. By default it is set to the built-in parser, and the module\n    name is equal to `Floki.HTMLParser.Mochiweb`, or from the value of the\n    application env of the same name.\n\n    See https://github.com/philss/floki#alternative-html-parsers for more details.\n\n  * `:parser_args` - A list of options to the parser. This can be used to pass options\n    that are specific for a given parser. Defaults to an empty list.","ref":"Floki.html#parse_fragment/2-options"},{"type":"function","title":"Floki.parse_fragment!/2","doc":"Parses a HTML fragment from a string.\n\nSimilar to `Floki.parse_fragment/1`, but raises `Floki.ParseError` if there was an\nerror parsing the fragment.","ref":"Floki.html#parse_fragment!/2"},{"type":"function","title":"Floki.raw_html/2","doc":"Converts HTML tree to raw HTML.\n\nNote that the resultant HTML may be different from the original one.\nSpaces after tags and doctypes are ignored.","ref":"Floki.html#raw_html/2"},{"type":"function","title":"Options - Floki.raw_html/2","doc":"* `:encode` - A boolean option to control if special HTML characters\n  should be encoded as HTML entities. Defaults to `true`.\n\n  You can also control the encoding behaviour at the application level via\n  `config :floki, :encode_raw_html, false`\n\n  * `:pretty` - Controls if the output should be formatted, ignoring\n  breaklines and spaces from the input and putting new ones in order\n  to pretty format the html. Defaults to `false`.","ref":"Floki.html#raw_html/2-options"},{"type":"function","title":"Examples - Floki.raw_html/2","doc":"iex> Floki.raw_html({\"div\", [{\"class\", \"wrapper\"}], [\"my content\"]})\n    ~s( my content )\n\n    iex> Floki.raw_html({\"div\", [{\"class\", \"wrapper\"}], [\"10 > 5\"]})\n    ~s( 10 &gt; 5 )\n\n    iex> Floki.raw_html({\"div\", [{\"class\", \"wrapper\"}], [\"10 > 5\"]}, encode: false)\n    ~s( 10 > 5 )\n\n    iex> Floki.raw_html({\"div\", [], [\"\\n   \", {\"span\", [], \"Fully indented\"}, \"    \\n\"]}, pretty: true)\n    \"\"\"\n     \n       \n        Fully indented\n       \n     \n    \"\"\"","ref":"Floki.html#raw_html/2-examples"},{"type":"function","title":"Floki.text/2","doc":"Returns the text nodes from a HTML tree.\n\nBy default, it will perform a deep search through the HTML tree.\nYou can disable deep search with the option `deep` assigned to false.\nYou can include content of script tags with the option `js` assigned to true.\nYou can specify a separator between nodes content.","ref":"Floki.html#text/2"},{"type":"function","title":"Options - Floki.text/2","doc":"* `:deep` - A boolean option to control how deep the search for\n    text is going to be. If `false`, only the level of the HTML node\n    or the first level of the HTML document is going to be considered.\n    Defaults to `true`.\n\n  * `:js` - A boolean option to control if the contents of script tags\n    should be considered as text. Defaults to `false`.\n\n  * `:sep` - A separator string that is added between text nodes.\n    Defaults to `\"\"`.\n\n  * `:include_inputs` - A boolean to control if ` ` or ` `\n    values should be included in the resultant string.\n    Defaults to `false`.\n\n  * `:html_parser` - The module of the backend that is responsible for parsing\n    the HTML string. By default it is set to `Floki.HTMLParser.Mochiweb`.","ref":"Floki.html#text/2-options"},{"type":"function","title":"Examples - Floki.text/2","doc":"iex> Floki.text({\"div\", [], [{\"span\", [], [\"hello\"]}, \" world\"]})\n    \"hello world\"\n\n    iex> Floki.text({\"div\", [], [{\"span\", [], [\"hello\"]}, \" world\"]}, deep: false)\n    \" world\"\n\n    iex> Floki.text({\"div\", [], [{\"script\", [], [\"hello\"]}, \" world\"]})\n    \" world\"\n\n    iex> Floki.text([{\"input\", [{\"type\", \"date\"}, {\"value\", \"2017-06-01\"}], []}], include_inputs: true)\n    \"2017-06-01\"\n\n    iex> Floki.text({\"div\", [], [{\"script\", [], [\"hello\"]}, \" world\"]}, js: true)\n    \"hello world\"\n\n    iex> Floki.text({\"ul\", [], [{\"li\", [], [\"hello\"]}, {\"li\", [], [\"world\"]}]}, sep: \"-\")\n    \"hello-world\"\n\n    iex> Floki.text([{\"div\", [], [\"hello world\"]}])\n    \"hello world\"\n\n    iex> Floki.text([{\"p\", [], [\"1\"]},{\"p\", [], [\"2\"]}])\n    \"12\"\n\n    iex> Floki.text({\"div\", [], [{\"style\", [], [\"hello\"]}, \" world\"]}, style: false)\n    \" world\"\n\n    iex> Floki.text({\"div\", [], [{\"style\", [], [\"hello\"]}, \" world\"]}, style: true)\n    \"hello world\"","ref":"Floki.html#text/2-examples"},{"type":"function","title":"Floki.traverse_and_update/2","doc":"Traverses and updates a HTML tree structure.\n\nThis function returns a new tree structure that is the result of applying the\ngiven `fun` on all nodes except text nodes.\nThe tree is traversed in a post-walk fashion, where the children are traversed\nbefore the parent.\n\nWhen the function `fun` encounters HTML tag, it receives a tuple with `{name,\nattributes, children}`, and should either return a similar tuple, a list of\ntuples to split current node or `nil` to delete it.\n\nThe function `fun` can also encounter HTML doctype, comment or declaration and\nwill receive, and should return, different tuple for these types. See the\ndocumentation for `t:html_comment/0`, `t:html_doctype/0` and\n`t:html_declaration/0` for details.\n\n**Note**: this won't update text nodes, but you can transform them when working\nwith children nodes.","ref":"Floki.html#traverse_and_update/2"},{"type":"function","title":"Examples - Floki.traverse_and_update/2","doc":"iex> html = [{\"div\", [], [\"hello\"]}]\n    iex> Floki.traverse_and_update(html, fn\n    ...>   {\"div\", attrs, children} -> {\"p\", attrs, children}\n    ...>   other -> other\n    ...> end)\n    [{\"p\", [], [\"hello\"]}]\n\n    iex> html = [{\"div\", [], [{:comment, \"I am comment\"}, {\"span\", [], [\"hello\"]}]}]\n    iex> Floki.traverse_and_update(html, fn\n    ...>   {\"span\", _attrs, _children} -> nil\n    ...>   {:comment, text} -> {\"span\", [], text}\n    ...>   other -> other\n    ...> end)\n    [{\"div\", [], [{\"span\", [], \"I am comment\"}]}]","ref":"Floki.html#traverse_and_update/2-examples"},{"type":"function","title":"Floki.traverse_and_update/3","doc":"Traverses and updates a HTML tree structure with an accumulator.\n\nThis function returns a new tree structure and the final value of accumulator\nwhich are the result of applying the given `fun` on all nodes except text nodes.\nThe tree is traversed in a post-walk fashion, where the children are traversed\nbefore the parent.\n\nWhen the function `fun` encounters HTML tag, it receives a tuple with\n`{name, attributes, children}` and an accumulator. It and should return a\n2-tuple like `{new_node, new_acc}`, where `new_node` is either a similar tuple\nor `nil` to delete the current node, and `new_acc` is an updated value for the\naccumulator.\n\nThe function `fun` can also encounter HTML doctype, comment or declaration and\nwill receive, and should return, different tuple for these types. See the\ndocumentation for `t:html_comment/0`, `t:html_doctype/0` and\n`t:html_declaration/0` for details.\n\n**Note**: this won't update text nodes, but you can transform them when working\nwith children nodes.","ref":"Floki.html#traverse_and_update/3"},{"type":"function","title":"Examples - Floki.traverse_and_update/3","doc":"iex> html = [{\"div\", [], [{:comment, \"I am a comment\"}, \"hello\"]}, {\"div\", [], [\"world\"]}]\n    iex> Floki.traverse_and_update(html, 0, fn\n    ...>   {\"div\", attrs, children}, acc ->\n    ...>     {{\"p\", [{\"data-count\", to_string(acc)} | attrs], children}, acc + 1}\n    ...>   other, acc -> {other, acc}\n    ...> end)\n    {[\n       {\"p\", [{\"data-count\", \"0\"}], [{:comment, \"I am a comment\"}, \"hello\"]},\n       {\"p\", [{\"data-count\", \"1\"}], [\"world\"]}\n     ], 2}\n\n    iex> html = {\"div\", [], [{\"span\", [], [\"hello\"]}]}\n    iex> Floki.traverse_and_update(html, [deleted: 0], fn\n    ...>   {\"span\", _attrs, _children}, acc ->\n    ...>     {nil, Keyword.put(acc, :deleted, acc[:deleted] + 1)}\n    ...>   tag, acc ->\n    ...>     {tag, acc}\n    ...> end)\n    {{\"div\", [], []}, [deleted: 1]}","ref":"Floki.html#traverse_and_update/3-examples"},{"type":"behaviour","title":"Floki.HTMLParser","doc":"A entry point to dynamic dispatch functions to\nthe configured HTML parser.\n\nThe configuration can be done with the `:html_parser`\noption when calling the functions, or for the `:floki` application:\n\n    Floki.parse_document(document, html_parser: Floki.HTMLParser.FastHtml)\n\nOr:\n\n    use Mix.Config\n    config :floki, :html_parser, Floki.HTMLParser.Mochiweb\n\nThe default parser is Mochiweb, which comes with Floki.\nYou can also choose between Html5ever or FastHtml.\n\nAnd it's possible to pass down options to the parsers using\nthe `parser_args` option.\n\nThis module is also a behaviour that those parsers must implement.","ref":"Floki.HTMLParser.html"},{"type":"function","title":"Floki.HTMLParser.parse_document/2","doc":"","ref":"Floki.HTMLParser.html#parse_document/2"},{"type":"callback","title":"Floki.HTMLParser.parse_document/2","doc":"","ref":"Floki.HTMLParser.html#c:parse_document/2"},{"type":"callback","title":"Floki.HTMLParser.parse_document_with_attributes_as_maps/2","doc":"","ref":"Floki.HTMLParser.html#c:parse_document_with_attributes_as_maps/2"},{"type":"function","title":"Floki.HTMLParser.parse_fragment/2","doc":"","ref":"Floki.HTMLParser.html#parse_fragment/2"},{"type":"callback","title":"Floki.HTMLParser.parse_fragment/2","doc":"","ref":"Floki.HTMLParser.html#c:parse_fragment/2"},{"type":"callback","title":"Floki.HTMLParser.parse_fragment_with_attributes_as_maps/2","doc":"","ref":"Floki.HTMLParser.html#c:parse_fragment_with_attributes_as_maps/2"},{"type":"task","title":"mix generate_entities","doc":"","ref":"Mix.Tasks.GenerateEntities.html"},{"type":"task","title":"mix generate_tokenizer_tests","doc":"It generates tests based on test files from WHATWG.\n\nThis task will take a look at tokenizer test files\nthat are located in \"./test/html5lib-tests/tokenizer\"\nand generate modules to run those tests.\n\nThis is necessary every time the specs of HTML change,\nso we can keep up to date and also we can keep track\nof what changed.","ref":"Mix.Tasks.GenerateTokenizerTests.html"},{"type":"exception","title":"Floki.ParseError","doc":"","ref":"Floki.ParseError.html"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](http://keepachangelog.com/en/1.0.0/)\nand this project adheres to [Semantic Versioning](http://semver.org/spec/v2.0.0.html).\n\n## [Unreleased][unreleased]\n\n## [0.37.1] - 2025-03-22","ref":"changelog.html"},{"type":"extras","title":"Fixed - Changelog","doc":"- Move regex declaration from module tag to inside function.\n  This is a fix to be compatible with the upcoming OTP 28.\n\n## [0.37.0] - 2024-12-06","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `Floki.css_escape/1` - thanks [@SteffenDE](https://github.com/SteffenDE).","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix bug propagating identity encoder in `raw_html/2` - thanks [@andyleclair](https://github.com/andyleclair).","ref":"changelog.html#fixed"},{"type":"extras","title":"Removed - Changelog","doc":"- Remove support for Elixir 1.13 and OTP 22.\n\n## [0.36.3] - 2024-10-21\n\nThis release contains some performance improvements, thanks to [@ypconstante](https://github.com/ypconstante).","ref":"changelog.html#removed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Stop `Floki.get_by_id/2` traversal on first match. Thanks [@ypconstante](https://github.com/ypconstante).\n\n- Remove extra whitespaces from nodes without attributes on `Floki.raw_html/1`.\n  Thank you [@ypconstante](https://github.com/ypconstante).\n\n- Fix `Floki.raw_html/1` typespecs. Thanks [@davydog187](https://github.com/davydog187).\n\n## [0.36.2] - 2024-04-26","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Implement the `Inspect` protocol for the `Floki.HTMLTree` struct.\n  This struct is currently private. Thank you [@vittoriabitton](https://github.com/vittoriabitton).","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix regression to respect config option `:encode` in `Floki.raw_html/2`.\n  Thanks [@Sgoettschkes](https://github.com/Sgoettschkes).\n\n- Make the `Floki.raw_html/2` treat the contents of the ` ` tag as plain text.\n  The idea is to align with `parse_document/2`.\n  Thank you [@aymanosman](https://github.com/aymanosman).\n\n## [0.36.1] - 2024-03-18","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix typespec of `get_by_id/2`.\n\n## [0.36.0] - 2024-03-01","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `Floki.get_by_id/1` that returns one element by ID or `nil`.\n  Thanks [@SteffenDE](https://github.com/SteffenDE).","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Improve options validation with `Keyword.validate!/2`.\n  This is not a change in APIs, but the error messages and opts validation\n  should be standardized now.\n  Thanks [@vittoriabitton](https://github.com/vittoriabitton).","ref":"changelog.html#changed"},{"type":"extras","title":"Removed - Changelog","doc":"- Drop support for Elixir v1.12.\n\n## [0.35.4] - 2024-02-19\n\nBesides the fix described below, this release also contains more performance\nimprovements, thanks to [@ypconstante](https://github.com/ypconstante).","ref":"changelog.html#removed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix order of results for `Floki.find/2`. This was a regression from the\n  previous version - thanks [@ypconstante](https://github.com/ypconstante).\n\n## [0.35.3] - 2024-01-25\n\nThis release has great performance improvements, thanks to the PRs\nfrom [@ypconstante](https://github.com/ypconstante)!\n\nMost of the main functions, such as `Floki.raw_html/2` and `Floki.find/2` are\nfaster and are using less memory. It's something like twice as fast, and half\nusage of memory for `find/2`, for example.","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Add `:leex` to Mix compilers. Fixes the build when running with dev version of Elixir.\n  Thanks [@wojtekmach](https://github.com/wojtekmach).\n\n- Fix `Floki.raw_html/2` when a tree using attributes as maps is given.\n  Thanks [@SupaMic](https://github.com/SupaMic).\n\n- Add a guard to `Floki.find/2` so people can have a better error message when an\n  invalid input is given. Thanks [@Hajto](https://github.com/Hajto).\n\n- Fix parsers to consider IO data as inputs. This may change in the next version\n  of Floki, as I plan to drop support for IO data.\n  Thanks [@ypconstante](https://github.com/ypconstante).","ref":"changelog.html#fixed"},{"type":"extras","title":"Removed - Changelog","doc":"- Remove outdated Gleam wrapper code. The external functions syntax in Gleam\n  has [changed](https://gleam.run/book/tour/external-functions.html). So now\n  the wrapper is not needed anymore.\n  Thanks [@michallepicki](https://github.com/michallepicki).\n\n## [0.35.2] - 2023-10-25","ref":"changelog.html#removed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Enable usage of IO data by removing a guard for binaries in the main\n  parser module.\n\n## [0.35.1] - 2023-10-16","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix a small regression of the mochiweb parser that was breaking when a\n  malformed HTML was used. For more details, see the original issue:\n  https://github.com/philss/floki/issues/492\n\n## [0.35.0] - 2023-10-13","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add support for parsing attributes as maps.\n\n  This makes `parse_document/2` and `parse_fragment/2` accept the option\n  `:attributes_as_maps` to change the behaviour and return attributes as\n  maps instead of lists of tuples.\n  The only parser that does not support it yet is the `fast_html`.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Drop support for Elixir v1.11.\n\n- Change the log level of parsing logger calls from \"info\" to \"debug\".\n  This will help to reduce the amount of noise in production apps.\n\n## [0.34.3] - 2023-06-02","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add boolean option `:include_inputs` to `Floki.text/2` that changes the result\n  of this function to include the values of inputs. So if there is any input with\n  a \"value\" attribute, we now include that value if this option is set to `true`.\n  Thanks [@viniciusmuller](https://github.com/viniciusmuller).","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix find of elements by classes that contain colons. This is useful for when \n  people are trying to find elements that contain Tailwind classes.\n  Thanks [@viniciusmuller](https://github.com/viniciusmuller).\n\n- Fix some typespecs that were using types from private modules. This is a fix\n  to the documentation.\n\n## [0.34.2] - 2023-02-24","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add option to pass down arguments to the parser in `Floki.parse_document/2` and\n  `Floki.parse_fragment/2`. Thanks [@Kuret](https://github.com/Kuret).\n\n- Add support for returning more elements from the `Floki.traverse_and_update/2` function callback.\n  This enables the creation of more elements in the tree, but should be used with care,\n  since the tree can grow a lot if the change is not controlled. Thanks [@martosaur](https://github.com/martosaur).\n\n## [0.34.1] - 2023-02-11","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix pseudo-class \":not\" selector parsing halting point.\n  This is a fix for when a \"pseudo-class\" \":not\" that contains an attribute selector is\n  followed by another selector. This is an example: \"a:not([class]), div\".\n\n- Ignore decimal numeric char ref when number is negative.\n\n## [0.34.0] - 2022-11-03","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- User configurable \"self-closing\" tags. Now it's possible to define which tags are considered\n\"self-closing\". Thanks [@inoas](https://github.com/inoas).","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Allow attribute values to not be escaped. This fixes `Floki.raw_html/2` when used with the\noption `encode: false`. Thanks [@juanazam](https://github.com/juanazam). \n- Fix `traverse_and_update/3` spec. Thanks [@WLSF](https://github.com/WLSF).","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Drop support for Elixir 1.9 and 1.10.\n- Remove `html_entities` dependency. We now use an internal encoder/decoder for entities.\n- Change the main branch name to `main`.\n\n## [0.33.1] - 2022-06-28","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Remove some warnings for unused code.\n\n## [0.33.0] - 2022-06-28","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add support for searching elements that contains text in a case-insensitive manner with\n`fl-icontains` - thanks [@nuno84](https://github.com/nuno84)","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Drop support for Elixir 1.8 and 1.9.\n- Fix and improve internal things - thanks [@derek-zhou](https://github.com/derek-zhou) and [@hissssst](https://github.com/hissssst)\n\n## [0.32.1] - 2022-03-24","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Allow root nodes to be selected using pseudo-classes - thanks [@rzane](https://github.com/rzane)\n\n## [0.32.0] - 2021-10-18","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add an HTML tokenizer written in Elixir - this still experimental and it's not stable API yet.\n- Add support for HTML IDs containing periods in the selectors - thanks [@Hugo-Hache](https://github.com/Hugo-Hache)\n- Add support for case-insensitive CSS attribute selectors - thanks [@fcapovilla](https://github.com/fcapovilla)\n- Add the `:root` pseudo-class selector - thanks [@fcapovilla](https://github.com/fcapovilla)\n\n## [0.31.0] - 2021-06-11","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Treat `style` and `title` tags as plaintext in Mochiweb - thanks [@SweetMNM](https://github.com/SweetMNM)\n\n## [0.30.1] - 2021-03-29","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix typespecs of `Floki.traverse_and_update/2` to make clear that it does not accept text nodes directly.\n\n## [0.30.0] - 2021-02-06","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add \":disabled\" pseudo selector - thanks [@vnegrisolo](https://github.com/vnegrisolo)\n- Add [Gleam](https://github.com/gleam-lang/gleam) adapter - thanks [@CrowdHailer](https://github.com/CrowdHailer)\n- Add pretty option to `Floki.raw_html/2` - thanks [@evaldobratti](https://github.com/evaldobratti)\n- Add `html_parser` option to `parse_` functions. This enables a more dynamic and functional\nconfiguration of the HTML parser in use.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Remove support for Elixir 1.7 - thanks [@carlosfrodrigues](https://github.com/carlosfrodrigues)\n- Replace `IO.warn` by `Logger.info` for deprecation warnings - thanks [@juulSme](https://github.com/juulSme)","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix typespecs for `find`, `attr` and `attribute` functions - thanks [@mtarnovan](https://github.com/mtarnovan)\n- Documentation Improvements - thanks [@kianmeng](https://github.com/kianmeng)\n\n## [0.29.0] - 2020-10-02","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `Floki.find_and_update/3` that updates nodes inside a tree, like traverse and update\nbut without allowing changes in the children nodes. There for the tree cannot grow in size,\nbut can have nodes removed.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Deprecate `Floki.map/2` because we have now `Floki.find_and_update/3` and `Floki.traverse_and_update/2` that\nare powerful APIs. `Floki.map/2` can be replaced by `Enum.map/2` as well - thanks [@josevalim](https://github.com/josevalim) for the idea!\n- Update optional dependency `fast_html` to `v2.0.4`","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix a bug when parsing a HTML with a XML inside using Mochiweb's parser","ref":"changelog.html#fixed"},{"type":"extras","title":"Improvements - Changelog","doc":"- Add more typespecs\n\n## [0.28.0] - 2020-08-26","ref":"changelog.html#improvements"},{"type":"extras","title":"Added - Changelog","doc":"- Add support for `:checked` pseudo-class selector - thanks [@wojtekmach](https://github.com/wojtekmach)","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Drop support for Elixir 1.6\n- Update version of `fast_html` to 2.0 in docs and CI - thanks [@rinpatch](https://github.com/rinpatch)","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix docs by mentioning HTML nodes supported for `traverse_and_update` - thanks [@hubertlepicki](https://github.com/hubertlepicki)\n\n## [0.27.0] - 2020-07-07","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- `Floki.filter_out/2` now can filter text nodes - thanks [@ckruse](https://github.com/ckruse)\n- Support more encoding entities in `Floki.raw_html/1` - thanks [@ntenczar](https://github.com/ntenczar)","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix `Floki.attribute/2` when there is only text nodes in the document - thanks [@ckruse](https://github.com/ckruse)","ref":"changelog.html#fixed"},{"type":"extras","title":"Improvements - Changelog","doc":"- Performance improvements of `Floki.raw_html/1` function - thanks [@josevalim](https://github.com/josevalim)\n- Improvements in the docs and specs of `Floki.traverse_and_update/2` and `Floki.children/1` - thanks [@josevalim](https://github.com/josevalim)\n- Improvements in the spec of `Floki.traverse_and_update/2` - thanks [@Dalgona](https://github.com/Dalgona)\n- Improve the CI setup to run the formatter correctly - thanks [@Cleidiano](https://github.com/Cleidiano)\n\n## [0.26.0] - 2020-02-17","ref":"changelog.html#improvements"},{"type":"extras","title":"Added - Changelog","doc":"- Add support for the pseudo-class selectors `:nth-last-child` and `:nth-last-of-type`","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix the typespecs of `Floki.traverse_and_update/3` - thanks [@RichMorin](https://github.com/RichMorin)","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Update optional dependency `fast_html` to `v1.0.3`\n\n## [0.25.0] - 2020-01-26","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `Floki.parse_fragment!/1` and `Floki.parse_document!/1` that has the same functionality of\nthe functions without the bang, but they return the document or fragment without the either tuple\nand will raise exception in case of errors - thanks [@schneiderderek](https://github.com/schneiderderek)\n- Add `Floki.traverse_and_update/3` which accepts an accumulator which is useful to keep\nthe state while traversing the HTML tree - thanks [@Dalgona](https://github.com/Dalgona)","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Update the `html_entities` dependency from `v0.5.0` to `v0.5.1`\n\n## [0.24.0] - 2020-01-01","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add support for [`fast_html`](https://hexdocs.pm/fast_html), which is a \"C Node\" wrapping\nLexborisov's [myhtml](https://github.com/lexborisov/myhtml) - thanks [@rinpatch](https://github.com/rinpatch)\n- Add setup to run our test suite against all parsers on CI - thanks [@rinpatch](https://github.com/rinpatch)\n- Add `Floki.parse_document/1` and `Floki.parse_fragment/1` in order to correct parse documents\nand fragments of documents - it also prevents the confusion and inconsistency of `parse/1`.\n- Configure `dialyxir` in order to run Dialyzer easily.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Deprecate `Floki.parse/1` and all the functions that uses it underneath. This means that all\nthe functions that accepted HTML as binary are deprecated as well. This includes `find/2`, `attr/4`,\n`filter_out/2`, `text/2` and `attribute/2`. The recommendation is to use those functions with an\nalready parsed document or fragment.\n- Remove support for `Elixir 1.5`.\n\n## [0.23.1] - 2019-12-01","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- It fixes the Mochiweb parser when there is an invalid charref.\n\n## [0.23.0] - 2019-09-11","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Remove Mochiweb as a hex dependency. It brings the code from the original project\nto Floki's codebase - thanks [@josevalim](https://github.com/josevalim)\n\n## [0.22.0] - 2019-08-21","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `Floki.traverse_and_update/2` that works in similar way to `Floki.map/2` but\ntraverse the tree and update the children elements. The difference from \"map\" is that\nthis function can create a tree with more or less nodes. - thanks [@ericlathrop](https://github.com/ericlathrop)","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Remove support for Elixir 1.4.\n\n## [0.21.0] - 2019-04-17","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add a possibility to filter `style` tags on `Floki.text/2` - thanks [@Vict0rynox](https://github.com/Vict0rynox)","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix `Floki.text/2` to consider the previous filter of `js` when filtering `style` - thanks [@Vict0rynox](https://github.com/Vict0rynox)\n- Fix typespecs for `Floki.filter_out/2` - thanks [@myfreeweb](https://github.com/myfreeweb)","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Drop support for Elixir 1.3 and below - thanks [@herbstrith](https://github.com/herbstrith)\n\n## [0.20.4] - 2018-09-24","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix `Floki.raw_html` to accept lists as attribute values - thanks [@katehedgpeth](https://github.com/katehedgpeth)\n\n## [0.20.3] - 2018-06-22","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix style and script tags with comments - thanks [@francois2metz](https://github.com/francois2metz)\n\n## [0.20.2] - 2018-05-09","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix `Floki.raw_html/1` to correct handle quotes and double quotes on attributes - thanks [@grych](https://github.com/grych)\n\n## [0.20.1] - 2018-04-05","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Remove `Enumerable.slice/1` compile warning for `Floki.HTMLTree` - thanks [@thecodeboss](https://github.com/thecodeboss)\n- Fix `Floki.find/2` that was failing on HTML that consists entirely of a comment - thanks [@ShaneWilton](https://github.com/ShaneWilton)\n\n## [0.20.0] - 2018-02-06","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Configurable raw_html/2 to allow optional encode of HTML entities - thanks [@davydog187](https://github.com/davydog187)","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix serialization of the tree after updating attribute - thanks [@francois2metz](https://github.com/francois2metz)\n\n## [0.19.3] - 2018-01-25","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Skip HTML entities encode for `Floki.raw_html/1` for `script` or `style` tags\n- Add `:html_entities` app to the list of OTP applications. It fixes production releases.\n\n## [0.19.2] - 2017-12-22","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- **(BREAKING CHANGE)** Re-encode HTML entities on `Floki.raw_html/1`.\n\n## [0.19.1] - 2017-12-04","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed doctype serialization for `Floki.raw_html/1` - thanks [@jhchen][https://github.com/jhchen]\n\n## [0.19.0] - 2017-11-11","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Added support for `nth-of-type`, `first-of-type`, `last-of-type` and `last-child` pseudo-classes - thanks [@saleem1337](https://github.com/saleem1337).\n- Added support for `nth-child` pseudo-class functional notation - thanks [@nirev](https://github.com/nirev).\n- Added functional notation support for `nth-of-type` pseudo-class.\n- Added a [Contributing guide](https://github.com/philss/floki/blob/main/CONTRIBUTING.md).","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Format all files according to the Elixir 1.6 formatter - thanks [@fcevado](https://github.com/fcevado).\n- Fix `Floki.raw_html` to support raw text - thanks [@craig-day](https://github.com/craig-day).\n\n## [0.18.1] - 2017-10-13","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Added a [Code of Conduct](https://github.com/philss/floki/blob/main/CODE_OF_CONDUCT.md).","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix XML tag when building HTML tree.\n- Return empty list when `Floki.filter_out/2` result is empty.\n\n## [0.18.0] - 2017-08-05","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Added `Floki.attr/4` that receives a function enabling manipulation of attribute values - thanks [@erikdsi](https://github.com/erikdsi).\n- Implement the String.Chars protocol for Floki.Selector.\n- Implement the Enumerable protocol for Floki.HTMLTree.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Changed `Floki.transform/2` to `Floki.map/2` and `Floki.Finder.apply_transform/2` to `Floki.Finder.map/2` - thanks [@aphillipo](https://github.com/aphillipo).","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix `Floki.raw_html/1` to consider XML prefixes - thanks [@sergey-kintsel](https://github.com/sergey-kintsel).\n- Fix `raw_html` for self closing tags with content - thanks [@navinpeiris](https://github.com/navinpeiris).","ref":"changelog.html#fixed"},{"type":"extras","title":"Removed - Changelog","doc":"- Removed support for Elixir 1.2.\n\n## [0.17.2] - 2017-05-25","ref":"changelog.html#removed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix attribute selectors in :not() - thanks [@jjcarstens](https://github.com/jjcarstens) and [@Eiji7](https://github.com/Eiji7)\n- Fix selector parser to consider combinators across selectors separated by commas.\nFor further details, please check the [pull request](https://github.com/philss/floki/pull/115) - thanks [@jjcarstens](https://github.com/jjcarstens) and [@mischov](https://github.com/mischov)\n\n## [0.17.1] - 2017-05-22","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix search when body has unencoded angles (`<` and `>`) - thanks [@sergey-kintsel](https://github.com/sergey-kintsel)\n- Fix crash caused by XML declaration inside body - thanks [@erikdsi](https://github.com/erikdsi)\n- Fix issue when finding fails if HTML begins with XML tag - thanks [@sergey-kintsel](https://github.com/sergey-kintsel)\n\n## [0.17.0] - 2017-04-12","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add support for multiple pseudo-selectors, line :not() and :nth-child() - thanks [@jjcarstens](https://github.com/jjcarstens)\n- Add support for multiple selectors inside the :not() pseudo-class selector - thanks [@jjcarstens](https://github.com/jjcarstens)\n\n## [0.16.0] - 2017-04-05","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Add support for selectors that only include a pseudo-class selector - thanks [@buhman](https://github.com/buhman)\n- Add support for a new selector: `fl-contains`, which returns elements that contains a given text - thanks [@buhman](https://github.com/buhman)","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix `:not()` pseudo-class selector to accept simple pseudo-class selectors as well - thanks [@mischov](https://github.com/mischov)\n\n## [0.15.0] - 2017-03-14","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Added support for the `:not()` pseudo-class selector.","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed pseudo-class selectors that are used in conjunction with combinators - thanks [@Eiji7](https://github.com/Eiji7)\n- Fixed order of elements after search using descendant combinator - thanks [@Eiji7](https://github.com/Eiji7)\n\n## [0.14.0] - 2017-02-07","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Added support for configuring `html5ever` as the HTML parser. Issue #83 - thanks [@hansihe](https://github.com/hansihe)\nand [@aphillipo](https://github.com/aphillipo)!\n\n## [0.13.2] - 2017-02-07","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed bug that was causing Floki.text/1 and Floki.filter_out/2\nto ignore \"trees\" with only text nodes. Issue #91 - thanks [@boydm](https://github.com/boydm).\n\n## [0.13.1] - 2017-01-22","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix ordering of duplicated descendant matches - thanks [@mmmries](https://github.com/mmmries)\n- Fix ordering of `Floki.text/1` when there are only root nodes - thanks [@mmmries](https://github.com/mmmries)\n\n## [0.13.0] - 2017-01-22","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Floki.filter_out/2 is now able to understand complex selectors to filter out from the tree.\n\n## [0.12.1] - 2017-01-20","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix search for elements using descendant combinator - issue #84 - thanks [@mmmries](https://github.com/mmmries)\n\n## [0.12.0] - 2016-12-28","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add basic support for nth-child pseudo-class selector.\nCloses [issue #64](https://github.com/philss/floki/issues/64).","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Remove support for Elixir 1.1 and below.\n- Remove public documentation for internal code.\n\n## [0.11.0] - 2016-10-12","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- First attempt to transform nodes with `Floki.transform/2`. It is not able to update\nthe tree yet, but works good with results from `Floki.find/2` - thanks [@bobjflong](https://github.com/bobjflong)","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Using Logger to notify unknown tokens in selector parser - thanks [@teamon](https://github.com/teamon) and [@geonnave](https://github.com/geonnave)\n- Replace `mochiweb_html` with `mochiweb` package. This is needed to fix conflict with other\npackages that are using `mochiweb`. - thanks [@aphillipo](https://github.com/aphillipo)\n\n## [0.10.1] - 2016-08-28","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix sibling search after immediate children - thanks [@gmile](https://github.com/gmile).\n\n## [0.10.0] - 2016-08-05","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Change the search for namespaced elements using the correct CSS3 syntax.","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix the search for child elements when is more than two elements deep - thanks [@gmile](https://github.com/gmile)\n\n## [0.9.0] - 2016-06-16","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- A separator between text when getting text from nodes - thanks [@rochdi](https://github.com/rochdi).\n\n## [0.8.1] - 2016-05-20","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Support rendering boolean attributes on `Floki.raw_html/1` - thanks [@iamvery](https://github.com/iamvery).","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Update Mochiweb HTML parser dependency to version 2.15.0.\n\n## [0.8.0] - 2016-03-06","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add possibility to search tags with namespaces.\n- Accept `Floki.Selector` as parameter of `Floki.find/2` instead of only strings - thanks [@hansihe](https://github.com/hansihe).","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Using a smaller package with only the Mochiweb HTML parser.\n\n## [0.7.2] - 2016-02-23","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Replace ` ` nodes by newline (`\\n`) in `DeepText` - thanks [@maxneuvians](https://github.com/maxneuvians).\n- Allow `FilterOut` to filter special nodes, like `comment`.\n\n## [0.7.1] - 2015-11-14","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Ignore PHP scripts when finding nodes.\n\n## [0.7.0] - 2015-11-03","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add support for excluding script notes in `Floki.text`.\nBy default, it will exclude those nodes, but it can be enabled with\nthe flag `js: true` - thanks [@vikeri](https://github.com/vikeri)!","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix find for sibling nodes when the precedent selector match an element\nat the end of sibling list - fix [issue #39](https://github.com/philss/floki/issues/39)\n\n## [0.6.1] - 2015-10-11","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix the `Floki.raw_html/1` to build HTML comments properly.\n\n## [0.6.0] - 2015-10-07","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `Floki.raw_html/2`.\n\n## [0.5.0] - 2015-09-27","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Add the child combinator to `Floki.find/2`.\n- Add the adjacent sibling combinator to `Floki.find/2`.\n- Add the general adjacent sibling combinator to `Floki.find/2`.\n\n## [0.4.1] - 2015-09-18","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Ignoring other files that are not lexer files (\".xrl\") under `src/` directory\nin Hex package. This fixes a crash when compiling using OTP 17.5 on Mac OS X.\nHuge thanks to [@henrik](https://github.com/henrik) and [@licyeus](https://github.com/licyeus) that pointed the\n[issue](https://github.com/philss/floki/issues/24)!\n\n## [0.4.0] - 2015-09-17","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- A robust representation of selectors in order to enable queries using a mix of selector types,\nsuch as classes with attributes, attributes with types, classes with classes and so on.\nHere is a list with examples of what is possible now:\n  - `Floki.find(html, \"a.foo\")`\n  - `Floki.find(html, \"a.foo[data-action=post]\")`\n  - `Floki.find(html, \".foo.bar\")`\n  - `Floki.find(html, \"a.foo[href$='.org']\")`\nThanks to [@licyeus](https://github.com/licyeus) to point out the [issue](https://github.com/philss/floki/issues/18)!\n- Include `mochiweb` in the applications list at mix.exs - thanks [@EricDykstra](https://github.com/EricDykstra)","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- `Floki.find/2` will now return a list instead of tuple when searching only by IDs.\nFor now on, Floki should always return the results inside a list, even if it's an ID match.","ref":"changelog.html#changed"},{"type":"extras","title":"Removed - Changelog","doc":"- `Floki.find/2` does not accept tuples as selectors anymore.\nThis is because with the robust selectors representation, it won't be necessary to query directly using\ntuples or another data structures rather than string.\n\n## [0.3.3] - 2015-08-23","ref":"changelog.html#removed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix `Floki.find/2` when there is a non-HTML input.\nIt closes the [issue #17](https://github.com/philss/floki/issues/17)\n\n## [0.3.2] - 2015-06-27","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix `Floki.DeepText` when there is a comment inside nodes.\n\n## [0.3.1] - 2015-06-21","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix `Floki.find/2` to consider XML trees.\n\n## [0.3.0] - 2015-06-07","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add attribute equals selector. This feature enables the user to search using\nHTML attributes other than \"class\" or \"id\".\nE.g: `Floki.find(html, \"[data-model=user]\")` - [@nelsonr](https://github.com/nelsonr)\n\n## [0.2.1] - 2015-06-04","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix `parse/1` when parsing a part of HTML without a root node - [@antonmi](https://github.com/antonmi)\n\n## [0.2.0] - 2015-05-03","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Support HTML string when searching for attributes with `Floki.attribute/2`.\n- Option for `Floki.text/2` to disable deep search and use flat search instead.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Change `Floki.text/1` to perform a deep search of text nodes.\n- Consider doctests in the test suite.\n\n## [0.1.1] - 2015-03-25","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add CHANGELOG.md following the [Keep a changelog](http://keepachangelog.com/).","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Using MochiWeb as a hex dependency instead of embedded code.\nIt closes the [issue #5](https://github.com/philss/floki/issues/5)\n\n## [0.1.0] - 2015-02-15","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Descendant selectors, like \".class tag\" to Floki.find/2.\n- Multiple selection, like \".class1, .class2\" to Floki.find/2.\n\n## [0.0.5] - 2014-12-21","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- `Floki.text/1`, which returns all text in the same level\nof the parent element inside HTML.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Elixir version requirement from \"~> 1.0.0\" to \">= 1.0.0\".\n\n[unreleased]: https://github.com/philss/floki/compare/v0.37.1...HEAD\n[0.37.1]: https://github.com/philss/floki/compare/v0.37.0...v0.37.1\n[0.37.0]: https://github.com/philss/floki/compare/v0.36.3...v0.37.0\n[0.36.3]: https://github.com/philss/floki/compare/v0.36.2...v0.36.3\n[0.36.2]: https://github.com/philss/floki/compare/v0.36.1...v0.36.2\n[0.36.1]: https://github.com/philss/floki/compare/v0.36.0...v0.36.1\n[0.36.0]: https://github.com/philss/floki/compare/v0.35.4...v0.36.0\n[0.35.4]: https://github.com/philss/floki/compare/v0.35.3...v0.35.4\n[0.35.3]: https://github.com/philss/floki/compare/v0.35.2...v0.35.3\n[0.35.2]: https://github.com/philss/floki/compare/v0.35.1...v0.35.2\n[0.35.1]: https://github.com/philss/floki/compare/v0.35.0...v0.35.1\n[0.35.0]: https://github.com/philss/floki/compare/v0.34.3...v0.35.0\n[0.34.3]: https://github.com/philss/floki/compare/v0.34.2...v0.34.3\n[0.34.2]: https://github.com/philss/floki/compare/v0.34.1...v0.34.2\n[0.34.1]: https://github.com/philss/floki/compare/v0.34.0...v0.34.1\n[0.34.0]: https://github.com/philss/floki/compare/v0.33.1...v0.34.0\n[0.33.1]: https://github.com/philss/floki/compare/v0.33.0...v0.33.1\n[0.33.0]: https://github.com/philss/floki/compare/v0.32.1...v0.33.0\n[0.32.1]: https://github.com/philss/floki/compare/v0.32.0...v0.32.1\n[0.32.0]: https://github.com/philss/floki/compare/v0.31.0...v0.32.0\n[0.31.0]: https://github.com/philss/floki/compare/v0.30.1...v0.31.0\n[0.30.1]: https://github.com/philss/floki/compare/v0.30.0...v0.30.1\n[0.30.0]: https://github.com/philss/floki/compare/v0.29.0...v0.30.0\n[0.29.0]: https://github.com/philss/floki/compare/v0.28.0...v0.29.0\n[0.28.0]: https://github.com/philss/floki/compare/v0.27.0...v0.28.0\n[0.27.0]: https://github.com/philss/floki/compare/v0.26.0...v0.27.0\n[0.26.0]: https://github.com/philss/floki/compare/v0.25.0...v0.26.0\n[0.25.0]: https://github.com/philss/floki/compare/v0.24.0...v0.25.0\n[0.24.0]: https://github.com/philss/floki/compare/v0.23.1...v0.24.0\n[0.23.1]: https://github.com/philss/floki/compare/v0.23.0...v0.23.1\n[0.23.0]: https://github.com/philss/floki/compare/v0.22.0...v0.23.0\n[0.22.0]: https://github.com/philss/floki/compare/v0.21.0...v0.22.0\n[0.21.0]: https://github.com/philss/floki/compare/v0.20.4...v0.21.0\n[0.20.4]: https://github.com/philss/floki/compare/v0.20.3...v0.20.4\n[0.20.3]: https://github.com/philss/floki/compare/v0.20.2...v0.20.3\n[0.20.2]: https://github.com/philss/floki/compare/v0.20.1...v0.20.2\n[0.20.1]: https://github.com/philss/floki/compare/v0.20.0...v0.20.1\n[0.20.0]: https://github.com/philss/floki/compare/v0.19.3...v0.20.0\n[0.19.3]: https://github.com/philss/floki/compare/v0.19.2...v0.19.3\n[0.19.2]: https://github.com/philss/floki/compare/v0.19.1...v0.19.2\n[0.19.1]: https://github.com/philss/floki/compare/v0.19.0...v0.19.1\n[0.19.0]: https://github.com/philss/floki/compare/v0.18.1...v0.19.0\n[0.18.1]: https://github.com/philss/floki/compare/v0.18.0...v0.18.1\n[0.18.0]: https://github.com/philss/floki/compare/v0.17.2...v0.18.0\n[0.17.2]: https://github.com/philss/floki/compare/v0.17.1...v0.17.2\n[0.17.1]: https://github.com/philss/floki/compare/v0.17.0...v0.17.1\n[0.17.0]: https://github.com/philss/floki/compare/v0.16.0...v0.17.0\n[0.16.0]: https://github.com/philss/floki/compare/v0.15.0...v0.16.0\n[0.15.0]: https://github.com/philss/floki/compare/v0.14.0...v0.15.0\n[0.14.0]: https://github.com/philss/floki/compare/v0.13.2...v0.14.0\n[0.13.2]: https://github.com/philss/floki/compare/v0.13.1...v0.13.2\n[0.13.1]: https://github.com/philss/floki/compare/v0.13.0...v0.13.1\n[0.13.0]: https://github.com/philss/floki/compare/v0.12.1...v0.13.0\n[0.12.1]: https://github.com/philss/floki/compare/v0.12.0...v0.12.1\n[0.12.0]: https://github.com/philss/floki/compare/v0.11.0...v0.12.0\n[0.11.0]: https://github.com/philss/floki/compare/v0.10.1...v0.11.0\n[0.10.1]: https://github.com/philss/floki/compare/v0.10.0...v0.10.1\n[0.10.0]: https://github.com/philss/floki/compare/v0.9.0...v0.10.0\n[0.9.0]: https://github.com/philss/floki/compare/v0.8.1...v0.9.0\n[0.8.1]: https://github.com/philss/floki/compare/v0.8.0...v0.8.1\n[0.8.0]: https://github.com/philss/floki/compare/v0.7.2...v0.8.0\n[0.7.2]: https://github.com/philss/floki/compare/v0.7.1...v0.7.2\n[0.7.1]: https://github.com/philss/floki/compare/v0.7.0...v0.7.1\n[0.7.0]: https://github.com/philss/floki/compare/v0.6.1...v0.7.0\n[0.6.1]: https://github.com/philss/floki/compare/v0.6.0...v0.6.1\n[0.6.0]: https://github.com/philss/floki/compare/v0.5.0...v0.6.0\n[0.5.0]: https://github.com/philss/floki/compare/v0.4.1...v0.5.0\n[0.4.1]: https://github.com/philss/floki/compare/v0.4.0...v0.4.1\n[0.4.0]: https://github.com/philss/floki/compare/v0.3.3...v0.4.0\n[0.3.3]: https://github.com/philss/floki/compare/v0.3.2...v0.3.3\n[0.3.2]: https://github.com/philss/floki/compare/v0.3.1...v0.3.2\n[0.3.1]: https://github.com/philss/floki/compare/v0.3.0...v0.3.1\n[0.3.0]: https://github.com/philss/floki/compare/v0.2.1...v0.3.0\n[0.2.1]: https://github.com/philss/floki/compare/v0.2.0...v0.2.1\n[0.2.0]: https://github.com/philss/floki/compare/v0.1.1...v0.2.0\n[0.1.1]: https://github.com/philss/floki/compare/v0.1.0...v0.1.1\n[0.1.0]: https://github.com/philss/floki/compare/v0.0.5...v0.1.0\n[0.0.5]: https://github.com/philss/floki/compare/v0.0.3...v0.0.5","ref":"changelog.html#changed"},{"type":"extras","title":"Overview","doc":"[![Actions Status](https://github.com/philss/floki/workflows/CI/badge.svg?branch=main)](https://github.com/philss/floki/actions)\n[![Floki version](https://img.shields.io/hexpm/v/floki.svg)](https://hex.pm/packages/floki)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/floki/)\n[![Hex.pm](https://img.shields.io/hexpm/dt/floki.svg)](https://hex.pm/packages/floki)\n[![License](https://img.shields.io/hexpm/l/floki.svg)](https://github.com/philss/floki/blob/main/LICENSE)\n[![Last Updated](https://img.shields.io/github/last-commit/philss/floki.svg)](https://github.com/philss/floki/commits/main)\n\n \n\n**Floki is a simple HTML parser that enables search for nodes using CSS selectors**.\n\n[Check the documentation 📙](https://hexdocs.pm/floki).","ref":"readme.html"},{"type":"extras","title":"Usage - Overview","doc":"Take this HTML as an example:\n\n```html\n<!doctype html>\n \n \n   \n     Floki \n     Enables search using CSS selectors \n     Github page \n     philss \n   \n   Hex package \n \n \n```\n\nHere are some queries that you can perform (with return examples):\n\n```elixir\n{:ok, document} = Floki.parse_document(html)\n\nFloki.find(document, \"p.headline\")\n# => [{\"p\", [{\"class\", \"headline\"}], [\"Floki\"]}]\n\ndocument\n|> Floki.find(\"p.headline\")\n|> Floki.raw_html\n# =>  Floki \n```\n\nEach HTML node is represented by a tuple like:\n\n    {tag_name, attributes, children_nodes}\n\nExample of node:\n\n    {\"p\", [{\"class\", \"headline\"}], [\"Floki\"]}\n\nSo even if the only child node is the element text, it is represented inside a list.","ref":"readme.html#usage"},{"type":"extras","title":"Installation - Overview","doc":"Add Floki to your `mix.exs`:\n\n```elixir\ndefp deps do\n  [\n    {:floki, \"~> 0.37.0\"}\n  ]\nend\n```\n\nAfter that, run `mix deps.get`.\n\nIf you are running on [Livebook](https://livebook.dev) or a script, you can install with `Mix.install/2`:\n\n```elixir\nMix.install([\n  {:floki, \"~> 0.37.0\"}\n])\n```\n\nYou can check the [changelog](CHANGELOG.md) for changes.","ref":"readme.html#installation"},{"type":"extras","title":"Dependencies - Overview","doc":"Floki needs the `:leex` module in order to compile.\nNormally this module is installed with Erlang in a complete installation.\n\nIf you get this [\"module :leex is not available\"](https://github.com/philss/floki/issues/35) error message, you need to install the `erlang-dev` and `erlang-parsetools` packages in order get the `:leex` module. The packages names may be different depending on your OS.","ref":"readme.html#dependencies"},{"type":"extras","title":"Alternative HTML parsers - Overview","doc":"By default Floki uses a patched version of `mochiweb_html` for parsing fragments\ndue to its ease of installation (it's written in Erlang and has no outside dependencies).\n\nHowever one might want to use an alternative parser due to the following\nconcerns:\n\n- Performance - It can be [up to 20 times slower than the alternatives](https://hexdocs.pm/fast_html/readme.html#benchmarks) on big HTML\n  documents.\n- Correctness - in some cases `mochiweb_html` will produce different results\n  from what is specified in [HTML5 specification](https://html.spec.whatwg.org/).\n  For example, a correct parser would parse `    bold   text  `\n  as `{\"title\", [], [\"   bold   text \"]}` since content inside ` ` is\n  to be [treated as plaintext](https://html.spec.whatwg.org/#the-title-element).\n  Albeit `mochiweb_html` would parse it as `{\"title\", [], [{\"b\", [], [\" bold \"]}, \" text \"]}`.\n\nFloki supports the following alternative parsers:\n\n- `fast_html` - A wrapper for [lexbor](https://github.com/lexbor/lexbor). A pure C HTML parser.\n- `html5ever` - A wrapper for [html5ever](https://github.com/servo/html5ever) written in Rust, developed as a part of the Servo project.\n\n`fast_html` is generally faster, according to the\n[benchmarks](https://hexdocs.pm/fast_html/readme.html#benchmarks) conducted by\nits developers.\n\nYou can perform a benchmark by running the following:\n\n```sh\nsh benchs/extract.sh\nmix run benchs/parse_document.exs\n```\n\nExtracting the files is needed only once.\n\n#### Using `html5ever` as the HTML parser\n\nThis dependency is written with a NIF using [Rustler](https://github.com/rusterlium/rustler), but\nyou don't need to install anything to compile it thanks to [RustlerPrecompiled](https://hexdocs.pm/rustler_precompiled/).\n\n```elixir\ndefp deps do\n  [\n    {:floki, \"~> 0.37.0\"},\n    {:html5ever, \"~> 0.15.0\"}\n  ]\nend\n```\n\nRun `mix deps.get` and compiles the project with `mix compile` to make sure it works.\n\nThen you need to configure your app to use `html5ever`:\n\n```elixir\n# in config/config.exs\n\nconfig :floki, :html_parser, Floki.HTMLParser.Html5ever\n```\n\nNotice that you can pass the HTML parser as an option in `parse_document/2` and `parse_fragment/2`.\n\n#### Using `fast_html` as the HTML parser\n\nA C compiler, GNU\\Make and CMake need to be installed on the system in order to\ncompile lexbor.\n\nFirst, add `fast_html` to your dependencies:\n\n```elixir\ndefp deps do\n  [\n    {:floki, \"~> 0.37.0\"},\n    {:fast_html, \"~> 2.0\"}\n  ]\nend\n```\n\nRun `mix deps.get` and compiles the project with `mix compile` to make sure it works.\n\nThen you need to configure your app to use `fast_html`:\n\n```elixir\n# in config/config.exs\n\nconfig :floki, :html_parser, Floki.HTMLParser.FastHtml\n```","ref":"readme.html#alternative-html-parsers"},{"type":"extras","title":"More about Floki API - Overview","doc":"To parse a HTML document, try:\n\n```elixir\nhtml = \"\"\"\n   \n   \n      \n   \n   \n\"\"\"\n\n{:ok, document} = Floki.parse_document(html)\n# => {:ok, [{\"html\", [], [{\"body\", [], [{\"div\", [{\"class\", \"example\"}], []}]}]}]}\n```\n\nTo find elements with the class `example`, try:\n\n```elixir\nFloki.find(document, \".example\")\n# => [{\"div\", [{\"class\", \"example\"}], []}]\n```\n\nTo convert your node tree back to raw HTML (spaces are ignored):\n\n```elixir\ndocument\n|> Floki.find(\".example\")\n|> Floki.raw_html\n# =>    \n```\n\nTo fetch some attribute from elements, try:\n\n```elixir\nFloki.attribute(document, \".example\", \"class\")\n# => [\"example\"]\n```\n\nYou can get attributes from elements that you already have:\n\n```elixir\ndocument\n|> Floki.find(\".example\")\n|> Floki.attribute(\"class\")\n# => [\"example\"]\n```\n\nIf you want to get the text from an element, try:\n\n```elixir\ndocument\n|> Floki.find(\".headline\")\n|> Floki.text\n\n# => \"Floki\"\n```","ref":"readme.html#more-about-floki-api"},{"type":"extras","title":"Supported selectors - Overview","doc":"Here you find all the [CSS selectors](https://www.w3.org/TR/selectors/#selectors) supported in the current version:\n\n| Pattern         | Description                  |\n|-----------------|------------------------------|\n| *               | any element                  |\n| E               | an element of type `E`       |\n| E[foo]          | an `E` element with a \"foo\" attribute |\n| E[foo=\"bar\"]    | an E element whose \"foo\" attribute value is exactly equal to \"bar\" |\n| E[foo~=\"bar\"]   | an E element whose \"foo\" attribute value is a list of whitespace-separated values, one of which is exactly equal to \"bar\" |\n| E[foo^=\"bar\"]   | an E element whose \"foo\" attribute value begins exactly with the string \"bar\" |\n| E[foo$=\"bar\"]   | an E element whose \"foo\" attribute value ends exactly with the string \"bar\" |\n| E[foo*=\"bar\"]   | an E element whose \"foo\" attribute value contains the substring \"bar\" |\n| E[foo\\|=\"en\"]    | an E element whose \"foo\" attribute has a hyphen-separated list of values beginning (from the left) with \"en\" |\n| E:nth-child(n)  | an E element, the n-th child of its parent |\n| E:nth-last-child(n)  | an E element, the n-th child of its parent, counting from bottom to up |\n| E:first-child   | an E element, first child of its parent |\n| E:last-child   | an E element, last child of its parent |\n| E:nth-of-type(n)  | an E element, the n-th child of its type among its siblings |\n| E:nth-last-of-type(n)  | an E element, the n-th child of its type among its siblings, counting from bottom to up |\n| E:first-of-type   | an E element, first child of its type among its siblings |\n| E:last-of-type   | an E element, last child of its type among its siblings |\n| E:checked       | An E element (checkbox, radio, or option) that is checked |\n| E:disabled      | An E element (button, input, select, textarea, or option) that is disabled |\n| E.warning       | an E element whose class is \"warning\" |\n| E#myid          | an E element with ID equal to \"myid\" (for ids containing periods, use `#my\\\\.id` or `[id=\"my.id\"]`) |\n| E:not(s)        | an E element that does not match simple selector s |\n| :root           | the root node or nodes (in case of fragments) of the document. Most of the times this is the `html` tag |\n| E F             | an F element descendant of an E element |\n| E > F           | an F element child of an E element |\n| E + F           | an F element immediately preceded by an E element |\n| E ~ F           | an F element preceded by an E element |\n\nThere are also some selectors based on non-standard specifications. They are:\n\n| Pattern               | Description                                                            |\n|-----------------------|------------------------------------------------------------------------|\n| E:fl-contains('foo')  | an E element that contains \"foo\" inside a text node                    |\n| E:fl-icontains('foo') | an E element that contains \"foo\" inside a text node (case insensitive) |","ref":"readme.html#supported-selectors"},{"type":"extras","title":"Suppressing log messages - Overview","doc":"Floki may log debug messages related to problems in the parsing of selectors, or parsing of the HTML tree.\nIt also may log some \"info\" messages related to deprecated APIs. If you want to suppress these log messages,\nplease consider setting the `:compile_time_purge_matching` option for `:logger` in your compile time configuration.\n\nSee https://hexdocs.pm/logger/Logger.html#module-compile-configuration for details.","ref":"readme.html#suppressing-log-messages"},{"type":"extras","title":"Special thanks - Overview","doc":"* [@arasatasaygin](https://github.com/arasatasaygin) for Floki's logo from the [Open Logos project](http://openlogos.org/).","ref":"readme.html#special-thanks"},{"type":"extras","title":"License - Overview","doc":"Copyright (c) 2014 Philip Sampaio Silva\n\nFloki is under MIT license. Check the [LICENSE](https://github.com/philss/floki/blob/main/LICENSE) file for more details.","ref":"readme.html#license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}